@@ -6,7 +6,7 @@
 
 # drawing functions -------------------------------------------------------
 
-rescale_slab_thickness = function(s_data, side, scale, orientation, justification, normalize, height, y, ymin, ymax) {
+rescale_slab_thickness = function(s_data, orientation, normalize, height, y, ymin, ymax) {
   # rescale the slab data to be within the confines of the bounding box
   # we do this *again* here (rather than in setup_data) because
   # position_dodge doesn't work if we only do it up there:
@@ -14,40 +14,50 @@ rescale_slab_thickness = function(s_data, side, scale, orientation, justificatio
   # all the same, and we want to preserve our normalization settings.
   # so we scale things based on the min height to ensure everything
   # is the same height
-  thickness_scale = scale * min(s_data[[height]])
-  y_scale = thickness_scale / s_data[[height]]
+  min_height = min(s_data[[height]])
 
-  switch_side(side, orientation,
-    topright = {
-      # the slight nudge of justification * s_data[[height]] * (1 - y_scale) ensures that
-      # justifications work properly when scale != 1 (and similarly for other values of `side`)
-      s_data[[y]] = s_data[[ymin]] + justification * s_data[[height]] * (1 - y_scale)
-      s_data[[ymin]] = s_data[[y]]
-      s_data[[ymax]] = s_data[[y]] + s_data$thickness * thickness_scale
-    },
-    bottomleft = {
-      s_data[[y]] = s_data[[ymax]] - (1 - justification) * s_data[[height]] * (1 - y_scale)
-      s_data[[ymin]] = s_data[[y]] - s_data$thickness * thickness_scale
-      s_data[[ymax]] = s_data[[y]]
-    },
-    both = {
-      s_data[[y]] = (s_data[[ymin]] + s_data[[ymax]]) / 2 - (0.5 - justification) * s_data[[height]] * (1 - y_scale)
-      s_data[[ymin]] = s_data[[y]] - s_data$thickness * thickness_scale / 2
-      s_data[[ymax]] = s_data[[y]] + s_data$thickness * thickness_scale / 2
+  # must do this within groups so that `side` can vary by slab
+  ddply_(s_data, c("group", y), function(d) {
+    scaling_aes = c("side", "justification", "scale")
+    for (a in scaling_aes) {
+      # use %in% here so that `NA`s are treated as equal
+      if (!isTRUE(all(d[[a]] %in% d[[a]][[1]]))) {
+        stop0(
+          "Slab `", a, "` cannot vary within groups:\n",
+          "all rows within the same slab must have the same `", a, "`."
+        )
+      }
     }
-  )
 
-  s_data
+    thickness_scale = d$scale * min_height
+
+    switch_side(d$side[[1]], orientation,
+      topright = {
+        d[[ymin]] = d[[y]] - d$justification * thickness_scale
+        d[[ymax]] = d[[y]] + (d$thickness - d$justification) * thickness_scale
+      },
+      bottomleft = {
+        d[[ymin]] = d[[y]] - (d$thickness - 1 + d$justification) * thickness_scale
+        d[[ymax]] = d[[y]] + (1 - d$justification) * thickness_scale
+      },
+      both = {
+        d[[ymin]] = d[[y]] - d$thickness * thickness_scale/2 + (0.5 - d$justification) * thickness_scale
+        d[[ymax]] = d[[y]] + d$thickness * thickness_scale/2 + (0.5 - d$justification) * thickness_scale
+      }
+    )
+
+    d
+  })
 }
 
 draw_slabs = function(self, s_data, panel_params, coord,
-  side, scale, orientation, justification, normalize, na.rm,
+  orientation, normalize, fill_type, na.rm,
   child_params
 ) {
   define_orientation_variables(orientation)
 
   s_data = self$override_slab_aesthetics(rescale_slab_thickness(
-    s_data, side, scale, orientation, justification, normalize, height, y, ymin, ymax
+    s_data, orientation, normalize, height, y, ymin, ymax
   ))
 
   # build groups for the slabs
@@ -58,17 +68,27 @@ draw_slabs = function(self, s_data, panel_params, coord,
     slab_grob = if (!is.null(d$fill) && !all(is.na(d$fill))) {
       # only bother drawing the slab if it has some fill colour to it
 
-      #split out slab data according to aesthetics that we want to be able to
-      # vary along the length of the slab, then assemble the top and bottom lines
-      # into a single entity
-      slab_data = group_slab_data_by(d, c("fill", "alpha"), orientation, side = "both")
-      GeomPolygon$draw_panel(transform(slab_data, colour = NA), panel_params, coord)
+      switch_fill_type(fill_type,
+        segments = {
+          # split out slab data according to aesthetics that we want to be able to
+          # vary along the length of the slab, then assemble the top and bottom lines
+          # into a single entity
+          slab_data = group_slab_data_by(d, c("fill", "alpha"), orientation, side = "both")
+          draw_polygon(transform(slab_data, colour = NA), panel_params, coord)
+        },
+        gradient = {
+          # build a linearGradient() representing the varying fill
+          gradient = make_gradient_fill(coord$transform(d, panel_params), orientation)
+          slab_data = group_slab_data_by(d, NULL, orientation, side = "both")
+          draw_polygon(transform(slab_data, colour = NA), panel_params, coord, fill = gradient)
+        }
+      )
     }
 
     if (!is.null(d$colour) && !all(is.na(d$colour))) {
       # we have an outline to draw around the outside of the slab:
       # the definition of "outside" depends on the value of `side`:
-      outline_data = group_slab_data_by(d, c("colour", "alpha", "size", "linetype"), orientation, side)
+      outline_data = group_slab_data_by(d, c("colour", "alpha", "size", "linetype"), orientation, d$side[[1]])
       gList(slab_grob, draw_path(outline_data, panel_params, coord))
     } else {
       slab_grob
@@ -76,7 +96,9 @@ draw_slabs = function(self, s_data, panel_params, coord,
   })
 
   # when side = "top" or "right", need to invert draw order so that overlaps happen in a sensible way
-  switch_side(side, orientation,
+  # unfortunately we can only do this by checking the first value of `side`, which
+  # means this may be incorrect if `side` varies across slabs
+  switch_side(s_data$side[[1]], orientation,
     topright = rev(slab_grobs),
     bottomleft = slab_grobs,
     both = slab_grobs
@@ -85,7 +107,7 @@ draw_slabs = function(self, s_data, panel_params, coord,
 
 
 draw_pointintervals = function(self, i_data, panel_params, coord,
-  orientation, justification, interval_size_domain, interval_size_range, fatten_point, show_point, na.rm,
+  orientation, interval_size_domain, interval_size_range, fatten_point, show_point, na.rm,
   child_params
 ) {
   if (nrow(i_data) == 0) return(list())
@@ -94,9 +116,6 @@ draw_pointintervals = function(self, i_data, panel_params, coord,
   interval_grobs = list()
   point_grobs = list()
 
-  # adjust y position based on justification
-  i_data[[y]] = i_data[[ymin]] + justification * i_data[[height]]
-
   if (nrow(i_data) > 0) {
     # reorder by interval width so largest intervals are drawn first
     i_data = i_data[order(abs(i_data[[xmax]] - i_data[[xmin]]), decreasing = TRUE),]
@@ -140,35 +159,35 @@ draw_path = function(data, panel_params, coord) {
 # aesthetic overrides -----------------------------------------------------
 
 override_slab_aesthetics = function(self, s_data) {
-  s_data$colour = s_data$slab_colour
-  s_data$fill = s_data$slab_fill %||% s_data$fill
-  s_data$fill = apply_colour_ramp(s_data$fill, s_data$fill_ramp)
-  s_data$alpha = s_data$slab_alpha %||% s_data$alpha
-  s_data$size = s_data$slab_size
-  s_data$linetype = s_data$slab_linetype %||% s_data$linetype
+  s_data$colour = s_data[["slab_colour"]]
+  s_data$fill = s_data[["slab_fill"]] %||% s_data[["fill"]]
+  s_data$fill = apply_colour_ramp(s_data[["fill"]], s_data[["fill_ramp"]])
+  s_data$alpha = s_data[["slab_alpha"]] %||% s_data[["alpha"]]
+  s_data$size = s_data[["slab_size"]]
+  s_data$linetype = s_data[["slab_linetype"]] %||% s_data[["linetype"]]
   s_data
 }
 
 override_point_aesthetics = function(self, p_data, size_domain, size_range, fatten_point) {
-  p_data$colour = p_data$point_colour %||% p_data$colour
-  p_data$colour = apply_colour_ramp(p_data$colour, p_data$colour_ramp)
-  p_data$fill = p_data$point_fill %||% p_data$fill
-  p_data$alpha = p_data$point_alpha %||% p_data$alpha
-  p_data$size = p_data$point_size %||% (fatten_point * get_line_size(p_data, size_domain, size_range))
+  p_data$colour = p_data[["point_colour"]] %||% p_data[["colour"]]
+  p_data$colour = apply_colour_ramp(p_data[["colour"]], p_data[["colour_ramp"]])
+  p_data$fill = p_data[["point_fill"]] %||% p_data[["fill"]]
+  p_data$alpha = p_data[["point_alpha"]] %||% p_data[["alpha"]]
+  p_data$size = p_data[["point_size"]] %||% (fatten_point * get_line_size(p_data, size_domain, size_range))
   p_data
 }
 
 override_interval_aesthetics = function(self, i_data, size_domain, size_range) {
-  i_data$colour = i_data$interval_colour %||% i_data$colour
-  i_data$colour = apply_colour_ramp(i_data$colour, i_data$colour_ramp)
-  i_data$alpha = i_data$interval_alpha %||% i_data$alpha
+  i_data$colour = i_data[["interval_colour"]] %||% i_data[["colour"]]
+  i_data$colour = apply_colour_ramp(i_data[["colour"]], i_data[["colour_ramp"]])
+  i_data$alpha = i_data[["interval_alpha"]] %||% i_data[["alpha"]]
   i_data$size = get_line_size(i_data, size_domain, size_range)
-  i_data$linetype = i_data$interval_linetype %||% i_data$linetype
+  i_data$linetype = i_data[["interval_linetype"]] %||% i_data[["linetype"]]
   i_data
 }
 
 get_line_size = function(i_data, size_domain, size_range) {
-  size = i_data$interval_size %||% i_data$size
+  size = i_data[["interval_size"]] %||% i_data[["size"]]
   pmax(
     (size - size_domain[[1]]) / (size_domain[[2]] - size_domain[[1]]) *
       (size_range[[2]] - size_range[[1]]) + size_range[[1]],
@@ -226,15 +245,6 @@ get_line_size = function(i_data, size_domain, size_range) {
 #' @eval rd_slabinterval_aesthetics()
 #' @inheritParams ggplot2::layer
 #' @param ...  Other arguments passed to [layer()].
-#' @param side Which side to draw the slab on. `"topright"`, `"top"`, and `"right"` are synonyms
-#' which cause the slab to be drawn on the top or the right depending on if `orientation` is `"horizontal"`
-#' or `"vertical"`. `"bottomleft"`, `"bottom"`, and `"left"` are synonyms which cause the slab
-#' to be drawn on the bottom or the left depending on if `orientation` is `"horizontal"` or
-#' `"vertical"`. `"topleft"` causes the slab to be drawn on the top or the left, and `"bottomright"`
-#' causes the slab to be drawn on the bottom or the right. `"both"` draws the slab mirrored on both
-#' sides (as in a violin plot).
-#' @param scale What proportion of the region allocated to this geom to use to draw the slab. If `scale = 1`,
-#' slabs that use the maximum range will just touch each other. Default is `0.9` to leave some space.
 #' @param orientation Whether this geom is drawn horizontally (`"horizontal"`) or
 #' vertically (`"vertical"`). The default, `NA`, automatically detects the orientation based on how the
 #' aesthetics are assigned, and should generally do an okay job at this. When horizontal (resp. vertical),
@@ -246,12 +256,6 @@ get_line_size = function(i_data, size_domain, size_range) {
 #' `"horizontal"` (tidybayes had an `orientation` parameter before ggplot did, and I think the tidybayes naming
 #' scheme is more intuitive: `"x"` and `"y"` are not orientations and their mapping to orientations is, in my
 #' opinion, backwards; but the base ggplot naming scheme is allowed for compatibility).
-#' @param justification Justification of the interval relative to the slab, where `0` indicates bottom/left
-#' justification and `1` indicates top/right justification (depending on `orientation`). If `justification`
-#' is `NULL` (the default), then it is set automatically based on the value of `side`: when `side` is
-#' `"top"`/`"right"` `justification` is set to `0`, when `side` is `"bottom"`/`"left"`
-#' `justification` is set to `1`, and when `side` is `"both"` `justification` is set to
-#' `0.5`.
 #' @param normalize How to normalize heights of functions input to the `thickness` aesthetic. If `"all"`
 #' (the default), normalize so that the maximum height across all data is `1`; if `"panels"`, normalize within
 #' panels so that the maximum height in each panel is `1`; if `"xy"`, normalize within
@@ -259,6 +263,13 @@ get_line_size = function(i_data, size_domain, size_range) {
 #' opposite axis is `1`; if `"groups"`, normalize within values of the opposite axis and within
 #' groups so that the maximum height in each group is `1`; if `"none"`, values are taken as is with no
 #' normalization (this should probably only be used with functions whose values are in \[0,1\], such as CDFs).
+#' @param fill_type What type of fill to use when the fill color or alpha varies within a slab. The default,
+#' `"segments"`, breaks up the slab geometry into segments for each unique combination of fill color and
+#' alpha value. This approach is supported by all graphics devices and works well for sharp cutoff values,
+#' but can result in ugly results if a large number of unique fill colors are being used (as in gradients,
+#' like in [`stat_gradientinterval()`]). When `fill_type == "gradient"`, a `linearGradient()` is used to
+#' create a smooth gradient fill. This works well for large numbers of unique fill colors, but requires
+#' R > 4.1 and is not yet supported on all graphics devices.
 #' @param interval_size_domain The minimum and maximum of the values of the size aesthetic that will be translated into actual
 #' sizes for intervals drawn according to `interval_size_range` (see the documentation for that argument.)
 #' @param interval_size_range (Deprecated). This geom scales the raw size aesthetic values when drawing interval and point sizes, as
@@ -312,11 +323,9 @@ geom_slabinterval = function(
   # 4. The argument definitions of GeomSlabinterval$draw_panel
   # This is needed to support how defaults work with child geoms,
   # amongst other things
-  side = c("topright", "top", "right", "bottomleft", "bottom", "left", "topleft", "bottomright", "both"),
-  scale = 0.9,
   orientation = NA,
-  justification = NULL,
   normalize = c("all", "panels", "xy", "groups", "none"),
+  fill_type = c("segments", "gradient"),
   interval_size_domain = c(1, 6),
   interval_size_range = c(0.6, 1.4),
   fatten_point = 1.8,
@@ -328,8 +337,8 @@ geom_slabinterval = function(
   show.legend = NA,
   inherit.aes = TRUE
 ) {
-  side = match.arg(side)
   normalize = match.arg(normalize)
+  fill_type = match.arg(fill_type)
 
   layer_geom_slabinterval(
     mapping = mapping,
@@ -339,11 +348,9 @@ geom_slabinterval = function(
     geom = GeomSlabinterval,
     ...,
 
-    side = side,
-    scale = scale,
     orientation = orientation,
-    justification = justification,
     normalize = normalize,
+    fill_type = fill_type,
     interval_size_domain = interval_size_domain,
     interval_size_range = interval_size_range,
     fatten_point = fatten_point,
@@ -439,7 +446,12 @@ GeomSlabinterval = ggproto("GeomSlabinterval", Geom,
     slab_fill = NULL,         # falls back to fill
     slab_alpha = NULL,        # falls back to alpha
     slab_linetype = NULL,     # falls back to linetype
-    fill_ramp = NULL
+    fill_ramp = NULL,
+
+    # scale and positioning aesthetics
+    side = "topright",
+    scale = 0.9,
+    justification = NULL
   ),
 
   # default aesthetics as they will actually be set (here or in the key)
@@ -463,18 +475,15 @@ GeomSlabinterval = ggproto("GeomSlabinterval", Geom,
     "ymin", "ymax", "xmin", "xmax", "width", "height", "thickness"
   ),
 
-  override_slab_aesthetics = override_slab_aesthetics,
-
-  override_point_aesthetics = override_point_aesthetics,
-
-  override_interval_aesthetics = override_interval_aesthetics,
+  # workaround (#84)
+  override_slab_aesthetics = function(self, ...) override_slab_aesthetics(self, ...),
+  override_point_aesthetics = function(self, ...) override_point_aesthetics(self, ...),
+  override_interval_aesthetics = function(self, ...) override_interval_aesthetics(self, ...),
 
   extra_params = c(
-    "side",
-    "scale",
     "orientation",
-    "justification",
     "normalize",
+    "fill_type",
     "interval_size_domain",
     "interval_size_range",
     "fatten_point",
@@ -485,11 +494,9 @@ GeomSlabinterval = ggproto("GeomSlabinterval", Geom,
   ),
 
   default_params = list(
-    side = "topright",
-    scale = 0.9,
     orientation = NA,
-    justification = NULL,
     normalize = "all",
+    fill_type = "segments",
     interval_size_domain = c(1, 6),
     interval_size_range = c(0.6, 1.4),
     fatten_point = 1.8,
@@ -499,8 +506,6 @@ GeomSlabinterval = ggproto("GeomSlabinterval", Geom,
     na.rm = FALSE
   ),
 
-  default_datatype = "slab",
-
   setup_params = function(self, data, params) {
     params = defaults(params, self$default_params)
 
@@ -522,7 +527,7 @@ GeomSlabinterval = ggproto("GeomSlabinterval", Geom,
     # fill it in with 0 so that we can still draw stuff
     data[[y]] = data[[y]] %||% 0
 
-    data$datatype = data$datatype %||% self$default_datatype
+    data$datatype = data[["datatype"]] %||% self$default_aes[["datatype"]]
 
     # normalize functions according to how we want to scale them
     switch(params$normalize,
@@ -558,33 +563,37 @@ GeomSlabinterval = ggproto("GeomSlabinterval", Geom,
     # figure out the bounding rectangles for each group
     # this is necessary so that the bounding box is correct for
     # positions to work (e.g. position_dodge, etc)
-    data[[height]] = data[[height]] %||% params[[height]] %||%
+    data[[height]] = params[[height]] %||% data[[height]] %||%
       resolution(data[[y]], FALSE)
 
     # determine bounding boxes based on justification: position
     # the min/max bounds around y such that y is at the correct
     # justification relative to the bounds
-    justification = get_justification(params$justification, params$side, params$orientation)
+    justification = get_justification(
+      params$justification %||% data[["justification"]],
+      params$side %||% data[["side"]] %||% self$default_aes$side,
+      params$orientation
+    )
     data[[ymin]] = data[[y]] - justification * data[[height]]
     data[[ymax]] = data[[y]] + (1 - justification) * data[[height]]
 
     data
   },
 
-  draw_key = draw_key_slabinterval_,
-  draw_key_slab = draw_key_slab_,
-  draw_key_point = draw_key_point_,
-  draw_key_interval = draw_key_interval_,
+  # workaround (#84)
+  draw_key = function(self, ...) draw_key_slabinterval_(self, ...),
+  draw_key_slab = function(self, ...) draw_key_slab_(self, ...),
+  draw_key_point = function(self, ...) draw_key_point_(self, ...),
+  draw_key_interval = function(self, ...) draw_key_interval_(self, ...),
 
-  draw_slabs = draw_slabs,
-  draw_pointintervals = draw_pointintervals,
+  # workaround (#84)
+  draw_slabs = function(self, ...) draw_slabs(self, ...),
+  draw_pointintervals = function(self, ...) draw_pointintervals(self, ...),
 
   draw_panel = function(self, data, panel_params, coord,
-      side = self$default_params$side,
-      scale = self$default_params$scale,
       orientation = self$default_params$orientation,
-      justification = self$default_params$justification,
       normalize = self$default_params$normalize,
+      fill_type = self$default_params$fill_type,
       interval_size_domain = self$default_params$interval_size_domain,
       interval_size_range = self$default_params$interval_size_range,
       fatten_point = self$default_params$fatten_point,
@@ -612,14 +621,14 @@ GeomSlabinterval = ggproto("GeomSlabinterval", Geom,
     # recover height: position_dodge adjusts ymax/ymin but not height
     data[[height]] = data[[ymax]] - data[[ymin]]
 
-    justification = get_justification(justification, side, orientation)
+    data$justification = get_justification(data[["justification"]], data[["side"]], orientation)
 
     slab_grobs = if (show_slab && !is.null(data$thickness)) {
       # thickness values were provided, draw the slabs
       s_data = data[data$datatype == "slab",]
       if (nrow(s_data) > 0) {
         self$draw_slabs(s_data, panel_params, coord,
-          side, scale, orientation, justification, normalize,
+          orientation, normalize, fill_type,
           na.rm, child_params
         )
       }
@@ -627,7 +636,7 @@ GeomSlabinterval = ggproto("GeomSlabinterval", Geom,
 
     point_interval_grobs = if (show_interval && !is.null(data[[xmin]]) && !is.null(data[[xmax]])) {
       self$draw_pointintervals(data[data$datatype == "interval",], panel_params, coord,
-        orientation, justification, interval_size_domain, interval_size_range, fatten_point, show_point, na.rm,
+        orientation, interval_size_domain, interval_size_range, fatten_point, show_point, na.rm,
         child_params
       )
     }
@@ -679,9 +688,33 @@ switch_side = function(side, orientation, topright, bottomleft, both) {
   )
 }
 
+# vectorized version of switch_side
+case_when_side = function(side, orientation, topright, bottomleft, both) {
+  # must make sure side and orientation are the same length as ifelse returns
+  # a result of the same length as the first arg only
+  common_length = max(length(side), length(orientation))
+  side = rep_len(side, length.out = common_length)
+  orientation = rep_len(orientation, length.out = common_length)
+
+  ifelse(
+    orientation %in% c("y", "horizontal"),
+    ifelse(
+      side %in% c("top", "topright", "topleft", "right"), topright, ifelse(
+      side %in% c("bottom", "bottomleft", "bottomright", "left"), bottomleft,
+      both
+    )),
+    # orientation is "vertical" or "x"
+    ifelse(
+      side %in% c("right", "topright", "bottomright", "top"), topright, ifelse(
+      side %in% c("left", "topleft", "bottomleft", "bottom"), bottomleft,
+      both
+    ))
+  )
+}
+
 get_justification = function(justification, side, orientation) {
   if (is.null(justification)) {
-    switch_side(side, orientation,
+    case_when_side(side, orientation,
       topright = 0,
       bottomleft = 1,
       both = 0.5
@@ -753,6 +786,84 @@ group_slab_data_by = function(slab_data, aesthetics = c("fill", "colour", "alpha
   )
 }
 
+#' construct a linearGradient() that can be used as a fill based on the fill
+#' and alpha aesthetics of the provided data
+#' @noRd
+make_gradient_fill = function(slab_data, orientation = "horizontal") {
+  define_orientation_variables(orientation)
+
+  x1 = paste0(x, "1")
+  y1 = paste0(y, "1")
+  x2 = paste0(x, "2")
+  y2 = paste0(y, "2")
+
+  x_min = min(slab_data[[x]])
+  x_max = max(slab_data[[x]])
+
+  gradient_args = list(
+    colours = alpha(slab_data$fill, slab_data$alpha),
+    stops = (slab_data[[x]] - x_min) / (x_max - x_min)
+  )
+
+  # x1 / x2 are relative to the bounds of the grob, hence 0 / 1
+  gradient_args[[x1]] = 0
+  gradient_args[[x2]] = 1
+  gradient_args[[y1]] = 0.5
+  gradient_args[[y2]] = 0.5
+
+  do.call(linearGradient, gradient_args)
+}
+
+#' draw a polygon grob --- based on ggplot2::GeomPolygon$draw_panel(), but
+#' allows for linearGradient fills
+#' @noRd
+draw_polygon = function(data, panel_params, coord, fill = NULL) {
+  n = nrow(data)
+
+  if (n == 1) return(zeroGrob())
+
+  munched = coord_munch(coord, data, panel_params)
+
+  # Sort by group to make sure that colors, fill, etc. come in same order
+  munched = munched[order(munched$group), ]
+
+  # For gpar(), there is one entry per polygon (not one entry per point).
+  # We'll pull the first value from each group, and assume all these values
+  # are the same within each group.
+  first_idx = !duplicated(munched$group)
+  first_rows = munched[first_idx, ]
+
+  ggname(
+    "geom_polygon",
+    polygonGrob(
+      munched$x, munched$y, default.units = "native",
+      id = munched$group,
+      gp = gpar(
+        col = first_rows$colour,
+        fill = fill %||% alpha(first_rows[["fill"]], first_rows[["alpha"]]),
+        lwd = first_rows$size * .pt,
+        lty = first_rows$linetype
+      )
+    )
+  )
+}
+
+switch_fill_type = function(fill_type, segments, gradient) {
+  if (getRversion() < "4.1.0" && fill_type == "gradient") {
+    message(                                                      # nocov
+      'fill_type = "gradient" is not supported in R < 4.1.0.\n',  # nocov
+      'Falling back to fill_type = "segments"'                    # nocov
+    )                                                             # nocov
+    fill_type = "segments"                                        # nocov
+  }
+
+  switch(fill_type,
+    segments = segments,
+    gradient = gradient,
+    stop("Unknown fill_type: ", deparse0(fill_type), '\nShould be "segments" or "gradient"')
+  )
+}
+
 
 # shortcut geoms ----------------------------------------------------------
 
@@ -801,13 +912,13 @@ GeomSlab = ggproto("GeomSlab", GeomSlabinterval,
 
   override_slab_aesthetics = function(self, s_data) {
     # we define these differently from geom_slabinterval to make this easier to use on its own
-    s_data$colour = s_data$slab_colour %||% s_data$colour
-    s_data$colour = apply_colour_ramp(s_data$colour, s_data$colour_ramp)
-    s_data$fill = s_data$slab_fill %||% s_data$fill
-    s_data$fill = apply_colour_ramp(s_data$fill, s_data$fill_ramp)
-    s_data$alpha = s_data$slab_alpha %||% s_data$alpha
-    s_data$size = s_data$slab_size %||% s_data$size
-    s_data$linetype = s_data$slab_linetype %||% s_data$linetype
+    s_data$colour = s_data[["slab_colour"]] %||% s_data[["colour"]]
+    s_data$colour = apply_colour_ramp(s_data[["colour"]], s_data[["colour_ramp"]])
+    s_data$fill = s_data[["slab_fill"]] %||% s_data[["fill"]]
+    s_data$fill = apply_colour_ramp(s_data[["fill"]], s_data[["fill_ramp"]])
+    s_data$alpha = s_data[["slab_alpha"]] %||% s_data[["alpha"]]
+    s_data$size = s_data[["slab_size"]] %||% s_data[["size"]]
+    s_data$linetype = s_data[["slab_linetype"]] %||% s_data[["linetype"]]
     s_data
   },
 
