@@ -10,25 +10,26 @@ deparse0 = function(expr, width.cutoff = 500, ...) {
   paste0(deparse(expr, width.cutoff = width.cutoff, ...), collapse = "")
 }
 
+stop0 = function(...) {
+  stop(..., call. = FALSE)
+}
+
+warning0 = function(...) {
+  warning(..., call. = FALSE)
+}
+
 # get all variable names from an expression
 # based on http://adv-r.had.co.nz/dsl.html
 all_names = function(x) {
   if (is.atomic(x)) {
     NULL
   } else if (is.name(x)) {
-    name = as.character(x)
-    if (name == "") {
-      NULL
-    }
-    else {
-      name
-    }
+    as.character(x)
   } else if (is.call(x) || is.pairlist(x)) {
     children = lapply(x[-1], all_names)
     unique(unlist(children))
   } else {
-    stop("Don't know how to handle type `", typeof(x), "`",
-      call. = FALSE)
+    stop0("Don't know how to handle type ", deparse0(typeof(x)))
   }
 }
 
@@ -45,11 +46,9 @@ defaults = function(x, defaults) {
   deprecated_args = intersect(old_names, names(enexprs(...)))
 
   if (length(deprecated_args) > 0) {
-    stop(
+    stop0(
       "\nIn ", fun, "(): The `", deprecated_args[[1]], "` argument is deprecated.\n",
-      message,
-
-      call. = FALSE
+      message
     )
   }
 }
@@ -61,13 +60,11 @@ defaults = function(x, defaults) {
     new_name = quo_name(enquo(new_arg))
     old_name = quo_name(enquo(old_arg))
 
-    warning(
+    warning0(
       "\nIn ", fun, "(): The `", old_name, "` argument is a deprecated alias for `",
       new_name, "`.\n",
       "Use the `", new_name, "` argument instead.\n",
-      "See help(\"tidybayes-deprecated\") or help(\"ggdist-deprecated\").\n",
-
-      call. = FALSE
+      "See help(\"tidybayes-deprecated\") or help(\"ggdist-deprecated\").\n"
     )
 
     old_arg
@@ -75,27 +72,89 @@ defaults = function(x, defaults) {
 }
 
 
-
 # workarounds -------------------------------------------------------------
 
-# workaround replacements for other patterns that don't quite do what we need them to
-# (especially when it comes to rvars...)
+# workarounds / replacements for common patterns
+
+#' @importFrom dplyr bind_rows
+map_dfr_ = function(data, fun, ...) {
+  # drop-in replacement for purrr::map_dfr
+  bind_rows(lapply(data, fun, ...))
+}
 
 pmap_dfr_ = function(data, fun) {
   # this is roughly equivalent to
   # pmap_dfr(df, function(...) { ... })
   # but works properly with vctrs (pmap_dfr seems broken on rvars?)
-  purrr::map_dfr(vctrs::vec_chop(data), function(row) do.call(fun, lapply(row, `[[`, 1)))
+  map_dfr_(vctrs::vec_chop(data), function(row) do.call(fun, lapply(row, `[[`, 1)))
 }
 
 ddply_ = function(data, groups, fun, ...) {
-  purrr::map_dfr(dplyr::group_split(data, dplyr::across(groups)), fun, ...)
+  bind_rows(dlply_(data, groups, fun, ...))
+}
+
+fct_explicit_na_ = function(x) {
+  x = as.factor(x)
+  if (anyNA(x)) {
+    na_name = "(Missing)"
+    levels_x = levels(x)
+    while (na_name %in% levels_x) {
+      na_name = paste0(na_name, "+")
+    }
+    levels(x) = c(levels_x, na_name)
+    x[is.na(x)] = na_name
+  }
+  x
 }
 
 dlply_ = function(data, groups, fun, ...) {
-  lapply(dplyr::group_split(data, dplyr::across(groups)), fun, ...)
+  # must make NAs explicit or they will be dropped by split()
+  group_vars = lapply(data[, groups, drop = FALSE], fct_explicit_na_)
+  # group_is = a list where each element is a numeric vector of indices
+  # corresponding to one group
+  group_is = if (length(group_vars) >= 1) {
+    unname(split(seq_len(nrow(data)), group_vars, drop = TRUE, lex.order = TRUE))
+  } else {
+    list(seq_len(nrow(data)))
+  }
+  lapply(group_is, function(group_i) {
+    # faster version of row_df = data[group_i, ]
+    row_df = tibble::new_tibble(lapply(data, `[`, group_i), nrow = length(group_i))
+    fun(row_df, ...)
+  })
 }
 
+map_dbl_ = function(X, FUN, ...) {
+  vapply(X, FUN, FUN.VALUE = numeric(1), ...)
+}
+
+map_lgl_ = function(X, FUN, ...) {
+  vapply(X, FUN, FUN.VALUE = logical(1), ...)
+}
+
+map2_chr_ = function(X, Y, FUN) {
+  as.character(mapply(FUN, X, Y, USE.NAMES = FALSE))
+}
+
+map2_dfr_ = function(X, Y, FUN) {
+  bind_rows(mapply(FUN, X, Y, SIMPLIFY = FALSE, USE.NAMES = FALSE))
+}
+
+iwalk_ = function(vec, fun, ...) {
+  # drop in replacement for purrr::iwalk()
+  nms = names(vec) %||% seq_along(x)
+  mapply(fun, vec, nms, MoreArgs = list(...))
+  invisible(vec)
+}
+
+fct_rev_ = function(x) {
+  if (is.character(x)) {
+    x = factor(x)
+  } else if (!is.factor(x)) {
+    stop0("`x` must be a factor (or character vector).")
+  }
+  factor(x, levels = rev(levels(x)), ordered = is.ordered(x))
+}
 
 
 # sequences ---------------------------------------------------------------
