@@ -1,7 +1,10 @@
 ## ----chunk_options, include=FALSE---------------------------------------------
-tiny_width = small_width = med_width = 6.75
-tiny_height = small_height = med_height = 4.5
-large_width = 8; large_height = 5.25
+tiny_width = 5.5
+tiny_height = 3 + 2/3
+small_width = med_width = 6.75
+small_height = med_height = 4.5
+large_width = 8
+large_height = 5.25
 
 knitr::opts_chunk$set(
   fig.width = small_width,
@@ -9,7 +12,8 @@ knitr::opts_chunk$set(
 )
 if (capabilities("cairo") && Sys.info()[['sysname']] != "Darwin") {
   knitr::opts_chunk$set(
-    dev.args = list(png = list(type = "cairo"))
+    dev = "png",
+    dev.args = list(type = "cairo")
   )
 }
 
@@ -40,7 +44,7 @@ df = tibble(
 ) %>%
   unnest(c(x, y))
 
-## ----spaghetti, fig.width = med_width, fig.height = med_height--------------------------------------------------------
+## ----spaghetti, fig.width = tiny_width, fig.height = tiny_height------------------------------------------------------
 df %>%
   filter(.draw %in% 1:100) %>%
   ggplot(aes(x = x, y = y, group = .draw)) +
@@ -51,14 +55,14 @@ df %>%
   group_by(x) %>%
   median_qi(y)
 
-## ----one_ribbon-------------------------------------------------------------------------------------------------------
+## ----one_ribbon, fig.width = tiny_width, fig.height = tiny_height-----------------------------------------------------
 df %>%
   group_by(x) %>%
   median_qi(y) %>%
   ggplot(aes(x = x, y = y, ymin = .lower, ymax = .upper)) +
-  geom_lineribbon()
+  geom_lineribbon(fill = "gray65")
 
-## ----geom_lineribbon, fig.width = med_width, fig.height = med_height--------------------------------------------------
+## ----geom_lineribbon, fig.width = tiny_width, fig.height = tiny_height------------------------------------------------
 df %>%
   group_by(x) %>%
   median_qi(y, .width = c(.50, .80, .95)) %>%
@@ -66,25 +70,25 @@ df %>%
   geom_lineribbon() +
   scale_fill_brewer()
 
-## ----stat_lineribbon, fig.width = med_width, fig.height = med_height--------------------------------------------------
+## ----stat_lineribbon, fig.width = tiny_width, fig.height = tiny_height------------------------------------------------
 df %>%
   ggplot(aes(x = x, y = y)) +
   stat_lineribbon() +
   scale_fill_brewer()
 
-## ----stat_lineribbon_width, fig.width = med_width, fig.height = med_height--------------------------------------------
+## ----stat_lineribbon_width, fig.width = tiny_width, fig.height = tiny_height------------------------------------------
 df %>%
   ggplot(aes(x = x, y = y)) +
   stat_lineribbon(.width = c(.66, .95)) +
   scale_fill_brewer()
 
-## ----stat_lineribbon_gradient, fig.width = med_width, fig.height = med_height-----------------------------------------
+## ----stat_lineribbon_gradient, fig.width = tiny_width, fig.height = tiny_height---------------------------------------
 df %>%
   ggplot(aes(x = x, y = y)) +
   stat_lineribbon(aes(fill = stat(.width)), .width = ppoints(50)) +
   scale_fill_distiller()
 
-## ----stat_lineribbon_gradient_ramp, fig.width = med_width, fig.height = med_height------------------------------------
+## ----stat_lineribbon_gradient_ramp, fig.width = tiny_width, fig.height = tiny_height----------------------------------
 df %>%
   ggplot(aes(x = x, y = y)) +
   stat_lineribbon(aes(fill_ramp = stat(.width)), .width = ppoints(50), fill = "#2171b5") +
@@ -96,18 +100,18 @@ df_2groups = rbind(
   mutate(df, g = "b", y = (y - 2) * 0.5)
 )
 
-## ----stat_lineribbon_2groups_brewer, fig.width = med_width, fig.height = med_height-----------------------------------
+## ----stat_lineribbon_2groups_brewer, fig.width = tiny_width, fig.height = tiny_height---------------------------------
 df_2groups %>%
   ggplot(aes(x = x, y = y, color = g)) +
   stat_lineribbon() +
   scale_fill_brewer()
 
-## ----stat_lineribbon_2groups_alpha, fig.width = med_width, fig.height = med_height------------------------------------
+## ----stat_lineribbon_2groups_alpha, fig.width = tiny_width, fig.height = tiny_height----------------------------------
 df_2groups %>%
   ggplot(aes(x = x, y = y, fill = g)) +
   stat_lineribbon(alpha = 1/4)
 
-## ----stat_lineribbon_2groups_ramp, fig.width = med_width, fig.height = med_height-------------------------------------
+## ----stat_lineribbon_2groups_ramp, fig.width = tiny_width, fig.height = tiny_height-----------------------------------
 df_2groups %>%
   ggplot(aes(x = x, y = y, fill = g)) +
   stat_lineribbon(aes(fill_ramp = stat(level)))
@@ -120,13 +124,13 @@ analytical_df = tibble(
 )
 analytical_df
 
-## ----stat_dist_lineribbon---------------------------------------------------------------------------------------------
+## ----stat_dist_lineribbon, fig.width = tiny_width, fig.height = tiny_height-------------------------------------------
 analytical_df %>%
   ggplot(aes(x = x, dist = dist_normal(y_mean, y_sd))) +
   stat_dist_lineribbon() +
   scale_fill_brewer()
 
-## ----curve_draws, fig.width = med_width, fig.height = med_height------------------------------------------------------
+## ----curve_draws, fig.width = tiny_width, fig.height = tiny_height----------------------------------------------------
 k = 11 # number of curves
 n = 501
 df = tibble(
@@ -141,7 +145,7 @@ df %>%
   ggplot(aes(x = x, y = y)) +
   geom_line(aes(group = .draw), alpha=0.2)
 
-## ----pointwise_ribbon, fig.width = med_width, fig.height = med_height-------------------------------------------------
+## ----pointwise_ribbon, fig.width = tiny_width, fig.height = tiny_height-----------------------------------------------
 df %>%
   group_by(x) %>%
   median_qi(y, .width = .5) %>%
@@ -151,7 +155,7 @@ df %>%
   scale_fill_brewer() +
   ggtitle("50% pointwise intervals with point_interval()")
 
-## ----curvewise_ribbon, fig.width = med_width, fig.height = med_height-------------------------------------------------
+## ----curvewise_ribbon, fig.width = tiny_width, fig.height = tiny_height, eval = requireNamespace("posterior", quietly = TRUE)----
 df %>%
   group_by(x) %>%
   curve_interval(y, .width = .5) %>%
@@ -161,7 +165,7 @@ df %>%
   scale_fill_brewer() +
   ggtitle("50% curvewise intervals with curve_interval()")
 
-## ----pointwise_curvewise, fig.width = med_width, fig.height = med_width-----------------------------------------------
+## ----pointwise_curvewise, fig.width = tiny_width, fig.height = tiny_width, eval = requireNamespace("posterior", quietly = TRUE)----
 k = 1000 # number of curves
 large_df = tibble(
     .draw = 1:k,
@@ -213,7 +217,7 @@ mtcars_boot = tibble(
   select(-m) %>%
   unnest(c(hp, mpg))
 
-## ----mtcars_spaghetti, fig.width = med_width, fig.height = med_height-------------------------------------------------
+## ----mtcars_spaghetti, fig.width = tiny_width, fig.height = tiny_height-----------------------------------------------
 mtcars_boot %>%
   filter(.draw < 400) %>%
   ggplot(aes(x = mpg, y = hp)) +
@@ -221,7 +225,7 @@ mtcars_boot %>%
   geom_point(data = mtcars) +
   coord_cartesian(ylim = c(0, 400))
 
-## ----mtcars_point_interval, fig.width = med_width, fig.height = med_height--------------------------------------------
+## ----mtcars_point_interval, fig.width = tiny_width, fig.height = tiny_height------------------------------------------
 mtcars_boot %>%
   ggplot(aes(x = mpg, y = hp)) +
   stat_lineribbon(.width = c(.5, .7, .9)) +
@@ -229,7 +233,7 @@ mtcars_boot %>%
   scale_fill_brewer() +
   coord_cartesian(ylim = c(0, 400))
 
-## ----mtcars_curve_interval, fig.width = med_width, fig.height = med_height--------------------------------------------
+## ----mtcars_curve_interval, fig.width = tiny_width, fig.height = tiny_height, eval = requireNamespace("posterior", quietly = TRUE)----
 mtcars_boot %>%
   group_by(mpg) %>%
   curve_interval(hp, .width = c(.5, .7, .9)) %>%
@@ -239,7 +243,7 @@ mtcars_boot %>%
   scale_fill_brewer() +
   coord_cartesian(ylim = c(0, 400))
 
-## ----mtcars_curve_interval_bd, fig.width = med_width, fig.height = med_height, eval = requireNamespace("fda", quietly = TRUE)----
+## ----mtcars_curve_interval_bd, fig.width = tiny_width, fig.height = tiny_height, eval = requireNamespace("fda", quietly = TRUE) && requireNamespace("posterior", quietly = TRUE)----
 mtcars_boot %>%
   group_by(mpg) %>%
   curve_interval(hp, .width = c(.5, .7, .9), .interval = "bd-mbd") %>%
