@@ -6,11 +6,12 @@
 library(dplyr)
 library(tidyr)
 
-context("curve_interval")
+
 
 
 test_that("curve_interval works with lineribbon", {
   skip_if_no_vdiffr()
+  skip_if_not_installed("posterior")
 
 
   k = 11 # number of curves
