@@ -6,7 +6,7 @@
 library(dplyr)
 library(distributional)
 
-context("stat_dist_")
+
 
 test_that("distribution eye plots work with the args aesthetic", {
   skip_if_no_vdiffr()
@@ -22,41 +22,41 @@ test_that("distribution eye plots work with the args aesthetic", {
 
   expect_warning(
     vdiffr::expect_doppelganger("vertical eye using args without na.rm",
-      p + stat_dist_eye(aes(x = dist), n = 40)
+      p + stat_dist_eye(aes(x = dist), n = 20)
     ),
     "Removed 2 rows containing missing values"
   )
 
   vdiffr::expect_doppelganger("vertical eye using args",
-    p + stat_dist_eye(aes(x = dist), na.rm = TRUE, n = 40)
+    p + stat_dist_eye(aes(x = dist), na.rm = TRUE, n = 20)
   )
 
   vdiffr::expect_doppelganger("horizontal eye using args",
-    p + stat_dist_eye(aes(y = dist), na.rm = TRUE, n = 40)
+    p + stat_dist_eye(aes(y = dist), na.rm = TRUE, n = 20)
   )
 
   vdiffr::expect_doppelganger("vertical half-eye using args",
-    p + stat_dist_halfeye(aes(x = dist), na.rm = TRUE, n = 40)
+    p + stat_dist_halfeye(aes(x = dist), na.rm = TRUE, n = 20)
   )
 
   vdiffr::expect_doppelganger("horizontal half-eye using args",
-    p + stat_dist_halfeye(aes(y = dist), na.rm = TRUE, n = 40)
+    p + stat_dist_halfeye(aes(y = dist), na.rm = TRUE, n = 20)
   )
 
   vdiffr::expect_doppelganger("ccdfinterval using args",
-    p + stat_dist_ccdfinterval(aes(x = dist), na.rm = TRUE, n = 40)
+    p + stat_dist_ccdfinterval(aes(x = dist), na.rm = TRUE, n = 25)
   )
 
   vdiffr::expect_doppelganger("ccdfintervalh using args",
-    p + stat_dist_ccdfinterval(aes(y = dist), na.rm = TRUE, n = 40)
+    p + stat_dist_ccdfinterval(aes(y = dist), na.rm = TRUE, n = 25)
   )
 
   vdiffr::expect_doppelganger("cdfinterval using args",
-    p + stat_dist_cdfinterval(aes(x = dist), na.rm = TRUE, n = 40)
+    p + stat_dist_cdfinterval(aes(x = dist), na.rm = TRUE, n = 25)
   )
 
   vdiffr::expect_doppelganger("cdfintervalh using args",
-    p + stat_dist_cdfinterval(aes(y = dist), na.rm = TRUE, n = 40)
+    p + stat_dist_cdfinterval(aes(y = dist), na.rm = TRUE, n = 25)
   )
 
 })
@@ -85,10 +85,20 @@ test_that("stat_dist_gradientinterval works", {
     scale_slab_alpha_continuous(range = c(0,1))
 
   vdiffr::expect_doppelganger("dist_gradientinterval with two groups",
-    p + stat_dist_gradientinterval(aes(x = dist), n = 20, p_limits = c(0.01, 0.99))
+    p + stat_dist_gradientinterval(aes(x = dist), n = 15, p_limits = c(0.01, 0.99), fill_type = "segments")
   )
   vdiffr::expect_doppelganger("dist_gradientintervalh with two groups",
-    p + stat_dist_gradientinterval(aes(y = dist), n = 20, p_limits = c(0.01, 0.99))
+    p + stat_dist_gradientinterval(aes(y = dist), n = 15, p_limits = c(0.01, 0.99), fill_type = "segments")
+  )
+
+  # N.B. the following two tests are currently a bit useless as vdiffr doesn't
+  # support linearGradient yet, but leaving them here so that once it does we
+  # have tests for this.
+  vdiffr::expect_doppelganger("fill_type = gradient with two groups",
+    p + stat_dist_gradientinterval(aes(x = dist), n = 15, p_limits = c(0.01, 0.99), fill_type = "gradient")
+  )
+  vdiffr::expect_doppelganger("fill_type = gradient with two groups, h",
+    p + stat_dist_gradientinterval(aes(y = dist), n = 15, p_limits = c(0.01, 0.99), fill_type = "gradient")
   )
 })
 
@@ -119,10 +129,10 @@ test_that("stat_dist_pointinterval, interval, and slab work", {
   )
 
   vdiffr::expect_doppelganger("dist_slab with two groups",
-    p + stat_dist_slab(aes(x = dist), n = 20)
+    p + stat_dist_slab(aes(x = dist), n = 15)
   )
   vdiffr::expect_doppelganger("dist_slabh with two groups",
-    p + stat_dist_slab(aes(y = dist), n = 20)
+    p + stat_dist_slab(aes(y = dist), n = 15)
   )
 })
 
@@ -141,11 +151,11 @@ test_that("scale transformation works", {
     scale_x_log10(breaks = 10^seq(-5,7, by = 2))
 
   vdiffr::expect_doppelganger("dist_halfeyeh log scale transform",
-    p_log + stat_dist_halfeye(n = 100)
+    p_log + stat_dist_halfeye(n = 20)
   )
 
   vdiffr::expect_doppelganger("dist_ccdfintervalh log scale transform",
-    p_log + stat_dist_ccdfinterval(n = 100)
+    p_log + stat_dist_ccdfinterval(n = 20)
   )
 
 
@@ -154,11 +164,11 @@ test_that("scale transformation works", {
     scale_x_reverse()
 
   vdiffr::expect_doppelganger("dist_halfeyeh reverse scale transform",
-    p_rev + stat_dist_halfeye(n = 100)
+    p_rev + stat_dist_halfeye(n = 40)
   )
 
   vdiffr::expect_doppelganger("ccdfinterval reverse scale transform",
-    p_rev + stat_dist_ccdfinterval(n = 100)
+    p_rev + stat_dist_ccdfinterval(n = 40)
   )
 })
 
@@ -208,7 +218,7 @@ test_that("pdf and cdf aesthetics work", {
     scale_slab_alpha_continuous(range = c(0,1))
 
   vdiffr::expect_doppelganger("pdf and cdf on a slabinterval",
-    p + stat_dist_slabinterval(aes(x = dist), n = 20, p_limits = c(0.01, 0.99))
+    p + stat_dist_slabinterval(aes(x = dist), n = 15, p_limits = c(0.01, 0.99))
   )
 })
 
@@ -224,11 +234,11 @@ test_that("distributional objects work", {
     ggplot(aes(x = name, dist = dist))
 
   vdiffr::expect_doppelganger("dist objects in stat_dist_halfeye",
-    p + stat_dist_halfeye(n = 20)
+    p + stat_dist_halfeye(n = 15)
   )
 
   vdiffr::expect_doppelganger("dist objects in stat_dist_ccdfinterval",
-    p + stat_dist_ccdfinterval(n = 20)
+    p + stat_dist_ccdfinterval(n = 15)
   )
 
   vdiffr::expect_doppelganger("dist_sample",
@@ -236,7 +246,7 @@ test_that("distributional objects work", {
       x = dist_sample(list(qnorm(ppoints(100)), qnorm(ppoints(100), mean = 1)))
     ) %>%
       ggplot(aes(dist = x, y = "a")) +
-      stat_dist_slab(fill = NA, color = "black", n = 20)
+      stat_dist_slab(fill = NA, color = "black", n = 15)
   )
 
 })
@@ -252,7 +262,7 @@ test_that("stat_dist_ works on factor dist names", {
     ggplot(aes(dist = x, y = y))
 
   vdiffr::expect_doppelganger("stat_dist_ with factor dist name",
-    p + stat_dist_slabinterval()
+    p + stat_dist_slabinterval(n = 15)
   )
 
 })
@@ -269,3 +279,107 @@ test_that("automatic finite limits work", {
     p + stat_dist_slab(n = 31)
   )
 })
+
+test_that("justification can vary", {
+  skip_if_no_vdiffr()
+
+  p = tribble(
+    ~id, ~name, ~dist,                ~just,
+    1, "norm", dist_normal(0, 1.5),  1,
+    2, "norm", dist_normal(0, 1),  0.5,
+    3, "t",    dist_student_t(3),    0
+  ) %>%
+    ggplot(aes(x = id, dist = dist, justification = just))
+
+  vdiffr::expect_doppelganger("ccdf with varying just",
+    p + stat_dist_ccdfinterval(n = 15)
+  )
+})
+
+test_that("NA distributional objects work", {
+  skip_if_no_vdiffr()
+
+
+  p = tribble(
+    ~name, ~dist,
+    "norm", dist_normal(0, 1.5),
+    "missing", NULL
+  ) %>%
+    ggplot(aes(x = name, dist = dist))
+
+  vdiffr::expect_doppelganger("NA dists in stat_dist_slabinterval",
+    p + stat_dist_halfeye(n = 15, na.rm = TRUE)
+  )
+
+  vdiffr::expect_doppelganger("NA dists in stat_dist_dotsinterval",
+    p + stat_dist_dotsinterval(n = 15, na.rm = TRUE)
+  )
+
+})
+
+test_that("stat_dist_ throws appropriate errors on ill-formed dists", {
+  expect_warning(
+    print(
+      tibble(y = c("a","b","c"), x = list(1,2,3)) %>%
+        ggplot(aes(y = y, dist = x)) + stat_dist_slabinterval()
+    ),
+    'The `dist` aesthetic does not support objects of type "numeric"'
+  )
+
+  expect_error(
+    distr_cdf(dist_normal(c(0,1))),
+    "distributional objects should never have length > 1 here"
+  )
+})
+
+
+# discrete distributions --------------------------------------------------
+
+test_that("stat_dist_ detects discrete distributions", {
+  p = tibble(lambda = c(13,7,2)) %>%
+    ggplot(aes(x = lambda))
+
+  vdiffr::expect_doppelganger("dist_poisson", {
+    p + stat_dist_halfeye(aes(dist = dist_poisson(lambda)), slab_color = "gray50")
+  })
+
+  vdiffr::expect_doppelganger("dist_poisson ccdf", {
+    p + stat_dist_ccdfinterval(aes(dist = dist_poisson(lambda)), slab_color = "gray50")
+  })
+
+  vdiffr::expect_doppelganger("dpois", {
+    p + stat_dist_halfeye(aes(dist = "pois", arg1 = lambda), slab_color = "gray50", outline_bars = TRUE)
+  })
+
+  vdiffr::expect_doppelganger("dpois ccdf", {
+    p + stat_dist_ccdfinterval(aes(dist = "pois", arg1 = lambda), slab_color = "gray50", outline_bars = TRUE)
+  })
+
+})
+
+
+# grouping order ----------------------------------------------------------
+
+test_that("stat_dist_ preserves existing grouping order", {
+  df = tribble(
+    ~Model, ~Parameter, ~Coefficient,  ~SE, ~linetype,
+    "C",       "MZ",         0.34, 0.07,    "dashed",
+    "C",     "Ereg",         0.28, 0.06,    "twodash",
+    "C",     "AE-Beta",      0.25, 0.06,    "solid",
+    "D",       "MZ",         0.31, 0.08,    "dashed"
+  )
+
+  # the labels should overlap the points exactly if grouping order is preserved
+  vdiffr::expect_doppelganger("grouped labels with pointintervals",
+    df %>%
+      ggplot() +
+      aes(x = Model, y = Coefficient,
+        label = Parameter, color = Parameter,
+        group = linetype,
+        dist = dist_normal(mu = Coefficient, sigma = SE)
+      ) +
+      stat_dist_pointinterval(position = "dodge") +
+      geom_label(position = position_dodge(width = 1))
+  )
+
+})
